--------------------------------------------------------------------------------
-- Slime Thrower by Holzchopf
-- v1.1 - 28 Sep 2011	- Updated hierarchy
--						- fixed math.random bug
--------------------------------------------------------------------------------

-- setup
if holzchopf==nil then holzchopf={} end
holzchopf.slimethrower={}
holzchopf.slimethrower.slime={}

-- load ressources
holzchopf.slimethrower.gfx_icn=loadgfx("holzchopf/slimeicon.png")
setmidhandle(holzchopf.slimethrower.gfx_icn)
holzchopf.slimethrower.gfx_wpn=loadgfx("weapons/paintgun.bmp")
setmidhandle(holzchopf.slimethrower.gfx_wpn)
holzchopf.slimethrower.gfx_pro=loadgfx("holzchopf/slime.png")
setmidhandle(holzchopf.slimethrower.gfx_pro)
holzchopf.slimethrower.sfx_att=loadsfx("paintgun.ogg")
holzchopf.slimethrower.sfx_hit=loadsfx("boing.ogg")

--------------------------------------------------------------------------------
-- Weapon
--------------------------------------------------------------------------------
holzchopf.slimethrower.id=addweapon("holzchopf.slimethrower","Slime Thrower",holzchopf.slimethrower.gfx_icn,0)
holzchopf.slimethrower.ammo=50
holzchopf.slimethrower.sndtimer=0

function holzchopf.slimethrower.draw()
	setblend(blend_alpha)
	setalpha(1)
	setcolor(255,255,255)
	drawinhand(holzchopf.slimethrower.gfx_wpn,6,0)
	if holzchopf.slimethrower.ammo>weapon_shots then
		hudammobar(holzchopf.slimethrower.ammo-weapon_shots,holzchopf.slimethrower.ammo)
		hudcrosshair(6,3)
	end
	-- sound timer
	holzchopf.slimethrower.sndtimer=holzchopf.slimethrower.sndtimer-1
end

function holzchopf.slimethrower.attack(attack)
	weapon_timer=weapon_timer-1
	if weapon_shots<holzchopf.slimethrower.ammo and weapon_timer<=0 and attack==1 then
		useweapon(0)
		weapon_shots=weapon_shots+1
		weapon_timer=5
		playsound(holzchopf.slimethrower.sfx_att)
		-- create projectiles
		id=createprojectile(holzchopf.slimethrower.slime.id)
		projectiles[id]={}
		randomseed(getframe()*29)
		rot=math.rad(getplayerrotation(0)+random(-2,2))
		projectiles[id].x=getplayerx(0)+(7*getplayerdirection(0))-math.sin(rot)*5.0
		projectiles[id].y=getplayery(0)+3+math.cos(rot)*5.0
		projectiles[id].sx=math.sin(rot)*10.0
		projectiles[id].sy=-math.cos(rot)*10.0
		randomseed(getframe()*31)
		projectiles[id].size=random(0.1,1)
		projectiles[id].smoke=math.random(1,10)
		-- end turn
		if weapon_shots>=holzchopf.slimethrower.ammo then
			endturn()
		end
	end
end

--------------------------------------------------------------------------------
-- Projectile
--------------------------------------------------------------------------------
holzchopf.slimethrower.slime.id=addprojectile("holzchopf.slimethrower.slime")
holzchopf.slimethrower.slime.sndtimer=0

function holzchopf.slimethrower.slime.draw(id)
	if projectiles[id].rotation==nil then
		setblend(blend_alpha)
		setalpha(1)
		setcolor(255,255,255)
		setscale(projectiles[id].size,projectiles[id].size)
		drawimage(holzchopf.slimethrower.gfx_pro,projectiles[id].x,projectiles[id].y)
	end
	-- garbace collection
	if getframesleft()<=0 and projectiles[id].timer==nil then
		projectiles[id].timer=math.random(25,75)
	elseif projectiles[id].timer~=nil then
		if projectiles[id].timer<=0 then
			if projectiles[id].rotation~=nil then
				if math.random(0,1)==1 then
					terrainalphaimage(gfx_blood25,projectiles[id].x,projectiles[id].y,0.5+math.random()*0.5,0,128,0)
				else
					terrainalphaimage(gfx_blood50,projectiles[id].x,projectiles[id].y,0.5+math.random()*0.5,0,128,0)
				end
			end
			if projectiles[id].smoke>3 then
				particle(p_smoke,projectiles[id].x,projectiles[id].y)
				particlesize(projectiles[id].size*10,projectiles[id].size*10)
				particlecolor(128,255,0)
				particlefadealpha(0.005)
			end
			freeprojectile(id)
			holzchopf.slimethrower.slime.sndtimer=0
		end
		projectiles[id].timer=projectiles[id].timer-1
	end
end

function holzchopf.slimethrower.slime.update(id)
	if projectiles[id].rotation==nil then
		holzchopf.slimethrower.slime.move(id)
	else
		holzchopf.slimethrower.slime.push(id)
	end
end

function holzchopf.slimethrower.slime.move(id)
	-- wind and gravity
	projectiles[id].sx=projectiles[id].sx+getwind()*0.5
	projectiles[id].sy=projectiles[id].sy+getgravity()
	-- Move (in substep loop for optimal collision precision)
	msubt=math.ceil(math.max(math.abs(projectiles[id].sx),math.abs(projectiles[id].sy)))
	msubx=projectiles[id].sx/msubt
	msuby=projectiles[id].sy/msubt
	for i=1,msubt,1 do
		-- collision check
		if collision(col3x3,projectiles[id].x+msubx,projectiles[id].y+msuby,1,0)==1 then
			-- define angle
			mx=0
			my=0
			for iy=-2,2,1 do
				for ix=-2,2,1 do
					if collision(col1x1,projectiles[id].x+ix,projectiles[id].y+iy,1,0)==1 then
						mx=mx+ix
						my=my+iy
					end
				end
			end
			projectiles[id].rotation=math.deg(math.atan2(mx,-my))
			-- paint
			setblend(blend_alpha)
			setalpha(1)
			setcolor(255,255,255)
			setscale(projectiles[id].size,projectiles[id].size)
			terrainalphaimage(holzchopf.slimethrower.gfx_pro,projectiles[id].x,projectiles[id].y)
			-- effects
			if holzchopf.slimethrower.sndtimer<=0 then
				playsound(holzchopf.slimethrower.sfx_hit)
				holzchopf.slimethrower.sndtimer=math.random(1,7)
			end
			break
		else
			-- move
			projectiles[id].x=projectiles[id].x+msubx
			projectiles[id].y=projectiles[id].y+msuby
			-- water
			if (projectiles[id].y)>getwatery()+5 then
				-- Effects
				particle(p_waterhit,projectiles[id].x,projectiles[id].y)
				if math.random(1,2)==1 then
					playsound(sfx_hitwater2)
				else
					playsound(sfx_hitwater3)
				end
				-- Free projectile
				freeprojectile(id)
				break
			end
		end
	end
end

function holzchopf.slimethrower.slime.push(id)
	-- every player
	players=playertable(0,1)
	for i=1,#players,1 do
		-- check distance
		dx=getplayerx(players[i])-projectiles[id].x
		dy=getplayery(players[i])-projectiles[id].y
		distance1=math.sqrt(dx*dx+dy*dy)
		dx=(getplayerx(players[i])+getplayerxspeed(players[i]))-projectiles[id].x
		dy=(getplayery(players[i])+getplayeryspeed(players[i]))-projectiles[id].y
		distance2=math.sqrt(dx*dx+dy*dy)
		if math.min(distance1,distance2)<=15 then
			rot=math.rad(projectiles[id].rotation+180)
			-- normal
			nx=math.sin(rot)
			ny=-math.cos(rot)
			-- player velocity
			vx=getplayerxspeed(players[i])*0.8
			vy=getplayeryspeed(players[i])*0.8
			-- project on normal
			f=(vx*nx+vy*ny)
			if f<=0 then
				px=f*nx
				py=f*ny
				-- flip velocity
				pvx=vx-2*px+nx*0.3
				pvy=vy-2*py+ny*0.3
				-- push
				playerpush(players[i],pvx,pvy,1)
				-- effects
				speed=math.sqrt(vx*vx+vy*vy)
				if speed>0.5 and holzchopf.slimethrower.slime.sndtimer<=getframe() then
					holzchopf.slimethrower.slime.sndtimer=getframe()+4
					playsound(sfx_splatter4)
				end
				particle(p_waterhit,projectiles[id].x,projectiles[id].y)
				particlecolor(128,255,0)
				particlerotation(math.deg(rot))
				if speed>1 then speed=1 end
				particlesize(0.8,0.8)
				particlealpha(speed)
				particlefadealpha(0.2)
			end
		end
	end
end